package ovh.gasser.newshapes.ui.menu;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ovh.gasser.newshapes.attributes.ColorAttributes;
import ovh.gasser.newshapes.attributes.SelectionAttributes;
import ovh.gasser.newshapes.shapes.SCollection;
import ovh.gasser.newshapes.shapes.Shape;
import ovh.gasser.newshapes.ui.Controller;
import ovh.gasser.newshapes.ui.ShapesView;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;


public class MenuEditListener implements ActionListener {
    private final static Logger logger = LoggerFactory.getLogger(MenuEditListener.class);

    private final SCollection model;
	private final ShapesView view;
	private final Controller controller;

	public MenuEditListener(SCollection model, ShapesView view, Controller controller) {
		this.model = model;
		this.view = view;
		this.controller = controller;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
        if (source instanceof JCheckBoxMenuItem item) {
            if (item.getText().equals("Draw border")) setBorder(item.isSelected());
            if (item.getText().equals("Fill Shape")) setShapeFilled(item.getState());
        } else {
            switch (e.getActionCommand()) {
                case "Change color" -> {
                    Color newColor = JColorChooser.showDialog(null, "Choose a color", Color.RED);
                    changeColor(newColor);
                }
                case "Change border color" -> {
                    Color newColor = JColorChooser.showDialog(null, "Choose a color", Color.RED);
                    changeBorderColor(newColor);
                }
                case "Delete" -> this.controller.deleteSelected();
            }
        }
		view.repaint();
	}

	private void changeColor(Color filledColor) {
        for (Shape s : model) {
            SelectionAttributes selAttrs = (SelectionAttributes) s.getAttributes(SelectionAttributes.ID);
            if ((selAttrs == null) || (! selAttrs.selected)) continue;
            ColorAttributes currentColAttrs = (ColorAttributes) s.getAttributes(ColorAttributes.ID);
            if (currentColAttrs == null) {
                logger.warn("No color attributes: {}", s);
                continue;
            }
            s.addAttributes(new ColorAttributes(true, currentColAttrs.stroked, filledColor, currentColAttrs.strokedColor));
        }
	}

	private void changeBorderColor(Color strockedColor) {
        for (Shape s : model) {
            SelectionAttributes selAttrs = (SelectionAttributes) s.getAttributes(SelectionAttributes.ID);
            if ((selAttrs == null) || (! selAttrs.selected)) continue;
            ColorAttributes currentColAttrs = (ColorAttributes) s.getAttributes(ColorAttributes.ID);
            if (currentColAttrs == null) {
                logger.warn("No color attributes: {}", s);
                continue;
            }
            s.addAttributes(new ColorAttributes(currentColAttrs.filled, true, currentColAttrs.filledColor, strockedColor));
        }
	}

	/*
	 * Change the state of the border of all selected shapes.
	 */
	private void setBorder(boolean state){
        for (Shape s : model) {
            SelectionAttributes selAttrs = (SelectionAttributes) s.getAttributes(SelectionAttributes.ID);
            if ((selAttrs == null) || (!selAttrs.selected)) continue;
            ColorAttributes colAttrs = (ColorAttributes) s.getAttributes(ColorAttributes.ID);
            if (colAttrs == null) {
                logger.warn("No color attributes: {}", s);
                continue;
            }
            s.addAttributes(new ColorAttributes(colAttrs.filled, state, colAttrs.filledColor, colAttrs.strokedColor));
        }
	}

	/*
	 * Change the filled state for all selected shapes.
	 */
	private void setShapeFilled(boolean state){
		logger.info("setShapeFilled(" + state + ")");
        for (Shape s : model) {
            SelectionAttributes selAttrs = (SelectionAttributes) s.getAttributes(SelectionAttributes.ID);
            if ((selAttrs == null) || (!selAttrs.selected)) continue;
            ColorAttributes colAttrs = (ColorAttributes) s.getAttributes(ColorAttributes.ID);
            if (colAttrs == null) {
                logger.warn("No color attributes: {}", s);
                continue;
            }
            s.addAttributes(new ColorAttributes(state, colAttrs.stroked, colAttrs.filledColor, colAttrs.strokedColor));
        }
	}
}
